@@ -1,7 +1,5 @@
-import os, platform, shutil
-from appscript import app as app_script, k
+import os, platform, shutil, math
 from sys import executable as python_executable, argv as python_argv, exit as python_exit
-from mactypes import Alias
 from pathlib import Path
 from subprocess import call
 from n4s import strgs
@@ -112,11 +110,15 @@ def copy_file(Source: Path, Destination: Path='', overwrite: bool=False, debug:
             return
 
 ## SEND MAIL
-def mail(Subject: str='', Send_To: str='', Body: str='', Attachment: Path=''):
+def mail(Send_To: str='', Subject: str='', Body: str='', Attachment: Path=''):
 
     ## SUPPORT FOR MACOS
     if system('is-mac'):
 
+        ## IMPORT LIBRARIES
+        from appscript import app as app_script, k
+        from mactypes import Alias
+
         ## CALL MAIL APP
         mail = app_script('Mail')
 
@@ -264,8 +266,35 @@ def path_exists(Path: Path, Make: bool=False, debug: bool=False):
                                 "Make sure path is type(list) or type(string), "
                                 "and that parent directories are created before nesting files\n") 
 
+## READS FILE SIZES
+def read_filesize(File: Path, Print: bool=False, Bytes: bool=False, debug: bool=False):
+    
+    file_size = os.path.getsize(File)
+
+    ## CAST FILE SIZE AS INT
+    file_size = int(file_size)
+    
+    ## CONVERT FILE SIZE
+    if file_size == 0:
+        return "0B"
+    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
+    i = int(math.floor(math.log(file_size, 1024)))
+    p = math.pow(1024, i)
+    s = round(file_size / p, 2)
+
+    # RETURN FILE SIZE IN BYTES
+    if Bytes:
+        if Print:
+            print(file_size)
+        return file_size
+    
+    ## RETURN FILE SIZE (FORMATTED)
+    if Print:
+        print("%s %s" % (s, size_name[i]))
+    return "%s %s" % (s, size_name[i])
+
 ## READS FILE EXTENSIONS
-def read_format(Input: str, Include_Period: bool=False, Print: bool=False, Uppercase: bool=False):
+def read_format(Input: str, Include_Period: bool=False, Print: bool=False, Uppercase: bool=False, Read_Filename: bool=False):
 
     ## INCLUDE PERIOD IN FORMAT
     if Include_Period:
@@ -285,6 +314,19 @@ def read_format(Input: str, Include_Period: bool=False, Print: bool=False, Upper
     if Uppercase:
         file_format = file_format.upper()
 
+    ## IF READ_FILENAME == ENABLED
+    if Read_Filename:
+
+        ## GET FILENAME
+        file_name = strgs.filter_text(Input.split('/')[-1], [file_format])
+
+        ## PRINT FORMAT TO TERMINAL
+        if Print:
+            print(file_name)
+
+        ## RETURN FORMAT
+        return file_name
+
     ## PRINT FORMAT TO TERMINAL
     if Print:
         print(file_format)
@@ -350,9 +392,89 @@ def remove_file(File: Path, debug: bool=False):
                 return print("\nn4s.fs.remove_file():\n"
                             f"Does Not Exist - {File}\n")
 
+## RENAME FILE / DIR
+def rename(Name: Path, Rename: str='', debug: bool=False):
+    '''
+    Name: Path to input file
+    Rename: str of the new file/dir name (do not use a full file path!)
+    debug: (boolean)
+    '''
+    ## VALIDATE INPUT PATH
+    if not path_exists(Name):
+        if debug:
+            return print("\nn4s.fs.rename():\n"
+                            f"Does Not Exist - {Name}\n")
+        return
+
+    ## LIST OF ROOT DIRS
+    root_dirs = [
+        str(root('user')),
+        str(root('desktop')),
+        str(root('docs')),
+        str(root('dl')),
+        str(root('applications')),
+        str(root('userlib')),
+        str(root('syslib')),
+        '/Users/afshari/test',
+        '/Users/afshari/Desktop/index'
+    ] 
+    
+    ## PREVENT RENAMING ROOT DIRS
+    for i in range(len(root_dirs)):
+        if Name == root_dirs[i] or Name == f"{root_dirs[i]}/":
+            if debug:
+                return print("\nn4s.fs.rename():\n"
+                            f"Cannot rename a user directory - {Name}\n")
+            return
+
+    ## CAPTURE INPUT PATH
+    Pathname = Name
+
+    ## CHECKS FOR TRAILING '/' DASH
+    if str(Name[-1]) == '/':
+        Name = str(Name.split('/')[-2])
+    else:
+        Name = str(Name.split('/')[-1])
+
+    ## UPDATE INPUT PATH
+    Pathname = strgs.filter_text(Pathname, [Name])
+    
+    ## VALIDATE RENAME ARG
+    if Rename == '':
+        if debug:
+            return print("\nn4s.fs.rename():\n"
+                        f"Rename arg can't be blank!\n")
+        return
+
+    ## CHECKS IF RENAME ALREADY EXISTS
+    if path_exists(f"{Pathname}{Rename}"):
+        if debug:
+            return print("\nn4s.fs.rename():\n"
+                            f"Rename already exists => {Rename}\n"
+                            f"{Pathname}{Rename}\n")
+        return
+
+    ## ASSIGN FULL PATH NAMES
+    Name = f"{Pathname}{Name}"
+    Rename = f"{Pathname}{Rename}"
+
+    ## CHECK FOR EXCPLICIT FILE EXTENSION
+    if not '.' in str(Rename).split('/')[-1]:
+        inherited_format = read_format(Name, True)
+        Rename = f"{Rename}{inherited_format}"
+
+    ## RUN RENAME
+    os.rename(Name, Rename)
+
+    ## DEBUG: PRINT RESULTS
+    if debug:
+        print("\nn4s.fs.rename():\n"
+                            f"Name => {Name}\n"
+                            f"Rename => {Rename}\n")
+
 ## FIND DIRECTORIES (ROOT == USER)
-def root(Dir: str='user', debug: bool=False):
-    if Dir == 'applications' or Dir == 'apps':
+def root(Directory: str='user', debug: bool=False):
+    if Directory == 'applications' or Directory == 'apps':
         if platform.system() == 'Darwin':
             if debug:
                 print("/Applications")
@@ -361,23 +483,23 @@ def root(Dir: str='user', debug: bool=False):
             if debug:
                 print("C:\Program Files")
             return "C:\Program Files"
-    if Dir == 'desktop' or Dir == 'desk':
+    if Directory == 'desktop' or Directory == 'desk':
         if debug:
             print(f"{Path.home()}/Desktop")
         return f"{Path.home()}/Desktop"
-    if Dir == 'documents' or Dir == 'docs':
+    if Directory == 'documents' or Directory == 'docs':
         if debug:
             print(f"{Path.home()}/Documents")
         return f"{Path.home()}/Documents"
-    if Dir == 'downloads' or Dir == 'dl':
+    if Directory == 'downloads' or Directory == 'dl':
         if debug:
             print(f"{Path.home()}/Downloads")
         return f"{Path.home()}/Downloads"
-    if Dir == 'user':
+    if Directory == 'user':
         if debug:
             print(Path.home())
         return Path.home()
-    if Dir == 'userlib':
+    if Directory == 'userlib':
         if platform.system() == 'Darwin':
             if debug:
                 print(f"{Path.home()}/Library")
@@ -386,7 +508,7 @@ def root(Dir: str='user', debug: bool=False):
             if debug:
                 print(f"{Path.home()}/AppData")
             return f"{Path.home()}/AppData"
-    if Dir == 'syslib':
+    if Directory == 'syslib':
         if platform.system() == 'Darwin':
             if debug:
                 print("/Library")
