@@ -1,6 +1,6 @@
-import os, shutil, re, base64, warnings, webbrowser, requests
+import os, shutil, re, base64, warnings, webbrowser, requests, math
 import http.client as httplib
-from n4s import fs
+from n4s import fs, strgs
 from bs4 import BeautifulSoup
 from pathlib import Path
 ##### Dismiss the 'XML' warning
@@ -9,6 +9,38 @@ category=UserWarning, module='bs4')
 #############################
 
 
+## WEB BROWSWER
+def browser(URL: str, Browser: str, Action: str, debug: bool=False):
+  '''
+  URL: (str) web address
+  Browser: (str) ['safari', 'chrome', 'firefox']
+  Action: (str) ['open', 'close', 'read']
+  debug: (bool) prints debug info to console
+  '''
+
+  ## OPEN WEB ADDRESS IN BROWSER
+  if Action.lower() == 'open':
+
+    ## SUPPORT FOR MACOS
+    if fs.system('is-mac') and Browser.lower() == 'safari':
+
+      ## IMPORT LIBRARIES
+      from appscript import app as app_script, k
+      from mactypes import Alias
+
+      ## OPEN URL IN BROWSER
+      web_browser = app_script(Browser)
+      web_browser.make(new=k.document,with_properties={k.URL:URL})
+    else:
+      ## PRINT ERROR
+      if debug:
+        print('\n\nn4s.web.browser()\nOnly Safari on macOS is supported at this time!\n')
+      return
+  else:
+    ## PRINT ERROR
+      if debug:
+        print("\n\nn4s.web.browser()\nOnly the 'open' command is supported at this time!\n")
+
 ## CREATE WEB FILES
 def build_html(Design: str='default', onefile: bool=False, Directory: Path=fs.root('desktop'), debug: bool=False):
         
@@ -626,6 +658,61 @@ def download(URL: str, Filename: str='', Save_Directory: Path=fs.root('downloads
     if debug:
       print(f'Done: {Save_Directory}/{Filename}')
 
+## READS FILE SIZES
+def read_filesize(File: Path, Print: bool=False, Bytes: bool=False, debug: bool=False):
+  '''
+  File: (Path) to the file you want read
+  Print: (bool) - prints the output to your terminal
+  Bytes: (bool) - return the size in bytes
+  debug: (bool) - print errors to terminal
+  '''
+
+  ## READ HEADER
+  try:
+    resp = requests.request('HEAD', File)
+  except Exception: ## INVALID ADDRESS / FAILED TO CONNECT
+    if debug:
+      print("\nn4s.web.read_filesize():\n"
+            "Unable to retrieve filesize\n"
+            "Error => Failed to Connect / Invalid Address\n")
+    return
+  
+  ## VERIFY CONTENT LENGTH KEY IN HEADER
+  if 'Content-Length' in resp.headers:
+      file_size = resp.headers['Content-Length']
+  else:
+      try:
+          resp = requests.request('HEAD', File)
+          file_size = resp.headers['Content-Length']
+      except KeyError:
+          if debug:
+            print("\nn4s.web.read_filesize():\n"
+                  "Unable to retrieve filesize\n"
+                  "Error => No 'Content-Length' Key\n")
+          return
+  
+  ## CAST FILE SIZE AS INT
+  file_size = int(file_size)
+  
+  ## CONVERT FILE SIZE
+  if file_size == 0:
+      return "0B"
+  size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
+  i = int(math.floor(math.log(file_size, 1024)))
+  p = math.pow(1024, i)
+  s = round(file_size / p, 2)
+  
+  # RETURN FILE SIZE IN BYTES
+  if Bytes:
+    if Print:
+      print(file_size)
+    return file_size
+  
+  ## RETURN FILE SIZE (FORMATTED)
+  if Print:
+    print("%s %s" % (s, size_name[i]))
+  return "%s %s" % (s, size_name[i])
+
 ## MERGE HTML/CSS/JS INTO ONE HTML FILE
 def merge_html(HTML: Path, CSS: Path, JS: Path, onefile: bool=False, debug: bool=False, filename: str='index', remove_webfiles: bool=False):
         '''
@@ -807,7 +894,13 @@ def strip_tags(Input: str):
 
         - Remove HTML tags from an input string
         '''
+        ## PREVENT CDATA CONTENT FROM BEING STRIPPED
+        Input = strgs.filter_text(Input, ['<![CDATA[', ']]>'])
+
+        ## STRIP HTML TAGS
         clean = re.compile('<.*?>')
+
+        ## RETURN (REMOVE WHITESPACE)
         return re.sub(clean, '', Input).strip()
 
 
